/*
* JBoss, Home of Professional Open Source
* Copyright 2006, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.metadata.spi.retrieval;

import java.util.concurrent.atomic.AtomicLong;
import java.io.Serializable;

/**
 * ValidTime.
 * 
 * @author <a href="adrian@jboss.com">Adrian Brock</a>
 * @version $Revision: 67232 $
 */
public class ValidTime implements Serializable
{
   /** The serialVersionUID */
   private static final long serialVersionUID = 1075815265759870744L;

   /** The next valid time */
   private static final AtomicLong next = new AtomicLong(Long.MIN_VALUE);
   
   /** This valid time */
   private volatile long value;
   
   /**
    * Create a new ValidTime.
    */
   public ValidTime()
   {
      value = next.get();
   }
   
   /**
    * Get the valid time
    * 
    * @return the valid time
    */
   public long getValidTime()
   {
      return value;
   }

   /**
    * Invalidate
    */
   public void invalidate()
   {
      value = next.incrementAndGet();
   }
}
